package ca.canadapost.cpcdp.ncshipping;

import java.math.BigDecimal;
import java.util.Map;

import ca.canadapost.cpcdp.ncshipping.stub.ncshipment.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.ncshipping.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;

/**
 * Sample code for the CreateNonContractShipment Canada Post service.
 * <p>
 * The CreateNonContractShipment service is used to generate and pay for a 
 * shipping label, including validation and payment authorization. 
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 */

public class CreateNCShipmentClient {
    // Main class for local testing only
    public static void main(String[] args) {    	
        try {
			// Your username, password and customer number are imported from the following file: CPCWS_SOAP_NCShipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            NCShipmentService service = new NCShipmentService();
            NCShipmentPortType port = service.getNCShipmentPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/ncshipment/v4");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //build request
            DomesticAddressDetailsType domesticAddressDetailsType = new DomesticAddressDetailsType();
            domesticAddressDetailsType.setAddressLine1("502 MAIN ST N");
            domesticAddressDetailsType.setCity("MONTREAL");
            domesticAddressDetailsType.setProvState("QC");
            domesticAddressDetailsType.setPostalZipCode("H2B1A0");

            SenderType senderType = new SenderType();
            senderType.setCompany("Capsule Corp.");
            senderType.setContactPhone("1 (514) 820 5879");
            senderType.setAddressDetails(domesticAddressDetailsType);

            DestinationAddressDetailsType destinationAddressDetailsType = new DestinationAddressDetailsType();
            destinationAddressDetailsType.setAddressLine1("123 Postal Drive");
            destinationAddressDetailsType.setCity("Ottawa");
            destinationAddressDetailsType.setProvState("ON");
            destinationAddressDetailsType.setCountryCode("CA");
            destinationAddressDetailsType.setPostalZipCode("K1P5Z9");

            DestinationType destinationType = new DestinationType();
            destinationType.setName("John Doe");
            destinationType.setCompany("ACME Corp.");
            destinationType.setAddressDetails(destinationAddressDetailsType);

            OptionType optionType = new OptionType();
            optionType.setOptionCode("DC");

            DeliverySpecType.Options options = new DeliverySpecType.Options();
            options.getOption().add(optionType);

            ParcelCharacteristicsType.Dimensions dimensions = new ParcelCharacteristicsType.Dimensions();
            dimensions.setLength(new BigDecimal(6));
            dimensions.setWidth(new BigDecimal(12));
            dimensions.setHeight(new BigDecimal(9));

            ParcelCharacteristicsType parcelCharacteristicsType = new ParcelCharacteristicsType();
            parcelCharacteristicsType.setWeight(new BigDecimal(15));
            parcelCharacteristicsType.setDimensions(dimensions);

            PreferencesType preferencesType = new PreferencesType();
            preferencesType.setShowPackingInstructions(true);

            ReferencesType referencesType = new ReferencesType();
            referencesType.setCostCentre("costCentre");
            referencesType.setCustomerRef1("customerReference1");
            referencesType.setCustomerRef2("customerReference2");

            DeliverySpecType deliverySpecType = new DeliverySpecType();
            deliverySpecType.setServiceCode("DOM.EP");
            deliverySpecType.setSender(senderType);
            deliverySpecType.setDestination(destinationType);
            deliverySpecType.setOptions(options);
            deliverySpecType.setParcelCharacteristics(parcelCharacteristicsType);
            deliverySpecType.setPreferences(preferencesType);
            deliverySpecType.setReferences(referencesType);

            NonContractShipmentType nonContractShipmentType = new NonContractShipmentType();
            nonContractShipmentType.setRequestedShippingPoint("H2B1A0");
            nonContractShipmentType.setDeliverySpec(deliverySpecType);

            CreateNonContractShipmentRequest request = new CreateNonContractShipmentRequest();
            request.setLocale("FR");
            request.setMailedBy(mailedBy);
            request.setNonContractShipment(nonContractShipmentType);

        	// Execute Request
        	CreateNonContractShipmentResponse response = port.createNCShipment(request);

            // Retrieve values from response object
            if (response.getMessages()  == null) {
            	System.out.println("Shipment Id: " + response.getNonContractShipmentInfo().getShipmentId());
            	for ( int i = 0; i < response.getNonContractShipmentInfo().getArtifacts().getArtifact().size(); i++) {
	                System.out.println("Artifact Id: " + response.getNonContractShipmentInfo().getArtifacts().getArtifact().get(i).getArtifactId());
	                System.out.println("Page Index: " + response.getNonContractShipmentInfo().getArtifacts().getArtifact().get(i).getPageIndex());
	                System.out.println();
            	}
            } else {
            	// Assume Error
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }                                    
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
