package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.stub.manifest.*;
import ca.canadapost.cpcdp.shipping.stub.manifest.AddressDetailsType;
import ca.canadapost.cpcdp.shipping.stub.manifest.ObjectFactory;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.shipping.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;

/**
 * Sample code for the TransmitShipments Canada Post service.
 * <p>
 * The TransmitShipments service is used to specify shipments to be included in a manifest.
 * Inclusion in a manifest is specified by group. Specific shipments may be excluded if
 * desired.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class TransmitShipmentsClient {
	// Main class for local testing only
    public static void main(String[] args) {
        try{
			// Your username and password are imported from the following file: CPCWS_SOAP_Shipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            ManifestService service = new ManifestService();
            ManifestPortType port = service.getManifestPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/manifest/v8");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //create request
            ObjectFactory of = new ObjectFactory();

            GroupIDListType groupIDListType = of.createGroupIDListType();
            groupIDListType.getGroupId().add("group");

            AddressDetailsType addressDetailsType = of.createAddressDetailsType();
            addressDetailsType.setAddressLine1("1230 Tako RD.");
            addressDetailsType.setCity("Ottawa");
            addressDetailsType.setProvState("ON");
            addressDetailsType.setCountryCode("CA");
            addressDetailsType.setPostalZipCode("H2B1A0");

            ManifestAddressType manifestAddressType  = of.createManifestAddressType();
            manifestAddressType.setManifestCompany("MajorShop");
            manifestAddressType.setPhoneNumber("514 829 5879");
            manifestAddressType.setAddressDetails(addressDetailsType);

            ShipmentTransmitSetType transmitSetType = of.createShipmentTransmitSetType();
            transmitSetType.setGroupIds(groupIDListType);
            transmitSetType.setRequestedShippingPoint("H2B1A0");
            transmitSetType.setCpcPickupIndicator(true);
            transmitSetType.setDetailedManifests(true);
            transmitSetType.setMethodOfPayment("account");
            transmitSetType.setManifestAddress(manifestAddressType);

            TransmitShipmentsRequest request = of.createTransmitShipmentsRequest();
            request.setMailedBy(mailedBy);
            request.setLocale("FR");
            request.setTransmitSet(transmitSetType);

            //execute request
            TransmitShipmentsResponse response = port.transmitShipments(request);

            //retrieve values from response
            if(response.getMessages() == null){
                if(response.getManifests() != null){
                    for(String id : response.getManifests().getManifestId()){
                        System.out.println("Manifest Id: " + id);
                    }
                }
            } else {
                //assume error
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
